/******************************************************************************
 *
 *    clnt_fattr.c,v : RPC to get the attributes of a shared file
 *    Copyright (C) 1995 A. Bode, S. Lamberts, T. Ludwig, C. R"oder
 *
 *    PFSLib (Parallel I/O on workstations)
 *
 *    PFSLib offers parallel access to files for a parallel application
 *    running on a cluster of workstations.
 *    It is intended but not restricted to be used in message passing
 *    applications based on PVM, NXLib, MPI, and other.
 *
 *    PFSLib consists of a LIBRARY, deamon PROGRAMS, and utility PROGRAMS.
 *
 *    PFSLib is free software; you can redistribute the LIBRARY and/or
 *    modify it under the terms of the GNU Library General Public
 *    License as published by the Free Software Foundation; either
 *    version 2 of the License, or (at your option) any later version.
 *    You can redistribute the daemon PROGRAMS and utility PROGRAMS
 *    and/or modify them under the terms of the GNU General Public
 *    License as published by the Free Software Foundation; either
 *    version 2 of the License, or (at your option) any later version.
 *
 *    PFSLib is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Library General Public License and GNU General Public License 
 *    for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License and the GNU General Public License along with this
 *    library; if not, write to the Free 
 *    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    Contact to the authors:
 *
 *    electronic mail: pfslib@informatik.tu-muenchen.de
 *
 *    paper mail:      Prof. Dr. A. Bode
 *                     Lehrstuhl f"ur Rechnertechnik und Rechnerorganisation
 *                     Institut f"ur Informatik
 *                     Technische Universit"at M"unchen
 *                     80290 M"unchen
 *                     Germany
 *
 *    This project was partially funded by a research grant form Intel
 *    Corporation. 
 *
 *****************************************************************************/

/******************************************************************************
 *
 *  RCS Filename : clnt_fattr.c,v
 *  RCS Date     : 1995/10/24 13:51:26
 *  RCS Revision : 1.4
 *  RCS Author   : lamberts
 *
 *  Authors: Stefan Lamberts, Christian R"oder
 *
 *****************************************************************************/
#ifndef lint
static char rcs_id[] = "clnt_fattr.c,v 1.4 1995/10/24 13:51:26 lamberts Rel";
#endif


#include <rpc/rpc.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>

#include "pfsd.h"
#include "clnt_defines.h"
#include "clnt_macros.h"
#include "pfslib_errno.h"

extern int _pfslib_maxfiles;
extern CLIENT * _pfslib_clnt;
extern pfslib_fhdl * _pfslib_ftbl;

int pfslib_stat
#ifdef ANSI_C
(int fd, struct pfslib_fattr *buf)
#else
(fd, buf)
int fd;
struct pfslib_fattr *buf;
#endif /* ANSI_C */
{
	fattrres  	*result;
  pfslib_fhdl  arg;
  
  fd -= VFDOFFSET;
  
  /* Check agruments */
  if ((fd < 0) || (fd >= _pfslib_maxfiles) ||
      (_pfslib_ftbl[fd].vfd == -1))
  {
    errno = EPFSLBADF;
    return (-1);
  }
  
  arg = _pfslib_ftbl[fd];

  if ((result = pfsd_getfattr_1(&arg,_pfslib_clnt)) == NULL )
  {
    clnt_perror(_pfslib_clnt, "call failed:");
    exit (1);
  }

  ERR_RESULT(fattrres_u, xdr_fattrres);  

  *buf   = result->fattrres_u.attr;

  xdr_free(xdr_fattrres, (char *)result);

	return (0);
}

